from config import (
    app_auth_access_token_type, app_auth_jwt_token_algorithm,
    app_auth_jwt_token_secret_key
)
from module.auth.core import AccessTokenUtil, JWTAccessTokenUtil


def init_token_util() -> AccessTokenUtil:
    if app_auth_access_token_type.lower() == 'jwt':
        return JWTAccessTokenUtil(
            secret_key=app_auth_jwt_token_secret_key,
            algorithm=app_auth_jwt_token_algorithm
        )
    raise ValueError(f'Invalid auth token type: {app_auth_access_token_type}')


access_token_util = init_token_util()



'''A Kubernetes Python Pulumi program to deploy kebab-zrb-app-name'''

from _common import MODE, BROKER_TYPE, ENABLE_MONITORING
from app_helper import (
    create_app_monolith_deployment, create_app_monolith_service,
    create_app_microservices_deployments, create_app_microservices_services
)
from helm_postgresql_helper import create_postgresql
from helm_rabbitmq_helper import create_rabbitmq
from helm_redpanda_helper import create_redpanda
from helm_signoz_helper import create_signoz

import pulumi

postgresql = create_postgresql()
pulumi.export('db', postgresql.resources)

if BROKER_TYPE == 'rabbitmq':
    rabbitmq = create_rabbitmq()
    pulumi.export('broker', rabbitmq.resources)
elif BROKER_TYPE == 'kafka':
    redpanda = create_redpanda()
    pulumi.export('broker', redpanda.resources)

if ENABLE_MONITORING:
    signoz = create_signoz()
    pulumi.export('signoz', signoz.resources)

if MODE == 'microservices':
    deployments = create_app_microservices_deployments()
    pulumi.export('app-deployment-names', [
        deployment.metadata['name'] for deployment in deployments
    ])
    services = create_app_microservices_services()
    pulumi.export('app-service-names', [
        service.metadata['name'] for service in services
    ])
else:
    deployment = create_app_monolith_deployment()
    pulumi.export('app-deployment-names', [deployment.metadata['name']])
    service = create_app_monolith_service()
    pulumi.export('app-service-names', [service.metadata['name']])



from typing import Any, Mapping
from abc import ABC
from sqlalchemy import Column, String
from sqlalchemy.orm import Session, relationship
from core.repo import Repo, DBEntityMixin, DBRepo
from module.auth.schema.group import Group, GroupData
from module.auth.component import Base
from module.auth.entity.table import group_permission
from module.auth.entity.permission.repo import DBEntityPermission


class DBEntityGroup(Base, DBEntityMixin):
    __tablename__ = "groups"
    name = Column(String)
    description = Column(String)
    permissions = relationship(
        "DBEntityPermission",
        secondary=group_permission
    )


class GroupRepo(Repo[Group, GroupData], ABC):
    pass


class GroupDBRepo(
    DBRepo[Group, GroupData], GroupRepo
):
    schema_cls = Group
    db_entity_cls = DBEntityGroup

    def _schema_data_to_db_entity_map(
        self, db: Session, group_data: GroupData
    ) -> Mapping[str, Any]:
        db_entity_map = super()._schema_data_to_db_entity_map(db, group_data)
        # Transform permissions
        db_entity_map['permissions'] = db.query(DBEntityPermission).filter(
            DBEntityPermission.id.in_(group_data.permissions)
        ).all()
        return db_entity_map



from zrb import CmdTask, runner, StrInput
from zrb.builtin.group import project_group

import os

CURRENT_DIR = os.path.dirname(__file__)
PROJECT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, '..', '..'))
RESOURCE_DIR = os.path.join(PROJECT_DIR, 'src', 'kebab-zrb-package-name')
PACKAGE_DIR = os.path.join(RESOURCE_DIR, 'src')

###############################################################################
# Task Definitions
###############################################################################

prepare_snake_zrb_package_name = CmdTask(
    name='prepare-kebab-zrb-package-name',
    description='Prepare venv for human readable zrb package name',
    group=project_group,
    cwd=RESOURCE_DIR,
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'prepare-venv.sh'),
    ]
)
runner.register(prepare_snake_zrb_package_name)

build_snake_zrb_package_name = CmdTask(
    name='build-kebab-zrb-package-name',
    description='Build human readable zrb package name',
    group=project_group,
    upstreams=[prepare_snake_zrb_package_name],
    cwd=RESOURCE_DIR,
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'build.sh'),
    ]
)
runner.register(build_snake_zrb_package_name)

publish_snake_zrb_package_name = CmdTask(
    name='publish-kebab-zrb-package-name',
    description='Publish human readable zrb package name',
    group=project_group,
    inputs=[
        StrInput(
            name='kebab-zrb-package-name-repo',
            prompt='Pypi repository for human readable zrb package name',
            description='Pypi repository for human readalbe zrb package name',
            default='pypi',
        )
    ],
    upstreams=[build_snake_zrb_package_name],
    cwd=RESOURCE_DIR,
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'publish.sh'),
    ]
)
runner.register(publish_snake_zrb_package_name)

install_snake_zrb_package_name_symlink = CmdTask(
    name='install-kebab-zrb-package-name-symlink',
    description='Install human readable zrb package name as symlink',
    group=project_group,
    upstreams=[build_snake_zrb_package_name],
    cwd=RESOURCE_DIR,
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'install-symlink.sh'),
    ]
)
runner.register(install_snake_zrb_package_name_symlink)



from zrb.helper.cli import create_cli
from zrb.helper.default_env import inject_default_env
from zrb.helper.log import logger
from zrb.helper.accessories.color import colored

inject_default_env()
logger.info(colored('Create CLI instance', attrs=['dark']))
cli = create_cli()

if __name__ == '__main__':
    cli()



from typing import AsyncIterator
from httpx import AsyncClient
from src.config import app_auth_admin_username, app_auth_admin_password
import pytest


@pytest.mark.asyncio
async def test_admin_user_login_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        login_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_response.status_code == 200
        json_login_response = login_response.json()
        assert json_login_response.get('token_type', '') == 'bearer'
        assert json_login_response.get('access_token', '') != ''


@pytest.mark.asyncio
async def test_admin_user_login_empty_identity_failed(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        login_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': '',
                'password': app_auth_admin_password
            }
        )
        assert login_response.status_code == 422


@pytest.mark.asyncio
async def test_admin_user_login_invalid_identity_failed(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        login_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': 'invalid-identity',
                'password': app_auth_admin_password
            }
        )
        assert login_response.status_code == 404


@pytest.mark.asyncio
async def test_admin_user_failed_invalid_password(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        login_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': 'invalid-password'
            }
        )
        assert login_response.status_code == 404


@pytest.mark.asyncio
async def test_create_normal_user_and_login_with_username_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )
        assert admin_access_token != ''
        # create user
        create_response = await client.post(
            '/api/v1/auth/users',
            json={
                'username': 'test-login-user-with-username-success',
                'password': 'test-password',
                'phone': '+628123456789',
                'email': 'test-login-user-with-username-success@test.com',
                'groups': [],
                'permissions': [],
                'description': ''
            },
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': 'test-login-user-with-username-success',
                'password': 'test-password'
            }
        )
        assert login_admin_response.status_code == 200
        json_login_response = login_admin_response.json()
        assert json_login_response.get('token_type', '') == 'bearer'
        assert json_login_response.get('access_token', '') != ''


@pytest.mark.asyncio
async def test_create_normal_user_and_login_with_phone_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )
        assert admin_access_token != ''
        # create user
        create_response = await client.post(
            '/api/v1/auth/users',
            json={
                'username': 'test-login-user-with-phone-success',
                'password': 'test-password',
                'phone': '+628123456789',
                'email': 'test-login-user-with-phone-success@test.com',
                'groups': [],
                'permissions': [],
                'description': ''
            },
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': '+628123456789',
                'password': 'test-password'
            }
        )
        assert login_admin_response.status_code == 200
        json_login_response = login_admin_response.json()
        assert json_login_response.get('token_type', '') == 'bearer'
        assert json_login_response.get('access_token', '') != ''


@pytest.mark.asyncio
async def test_create_normal_user_and_login_with_email_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )
        assert admin_access_token != ''
        # create user
        create_response = await client.post(
            '/api/v1/auth/users',
            json={
                'username': 'test-login-user-with-email-success',
                'password': 'test-password',
                'phone': '+628123456789',
                'email': 'test-login-user-with-email-success@test.com',
                'groups': [],
                'permissions': [],
                'description': ''
            },
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': 'test-login-user-with-email-success@test.com',
                'password': 'test-password'
            }
        )
        assert login_admin_response.status_code == 200
        json_login_response = login_admin_response.json()
        assert json_login_response.get('token_type', '') == 'bearer'
        assert json_login_response.get('access_token', '') != ''


@pytest.mark.asyncio
async def test_create_normal_user_and_login_failed(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )
        assert admin_access_token != ''
        # create user
        create_response = await client.post(
            '/api/v1/auth/users',
            json={
                'username': 'test-login-user-with-email-failed',
                'password': 'test-password',
                'phone': '+628123456789',
                'email': 'test-login-user-with-email-failed@test.com',
                'groups': [],
                'permissions': [],
                'description': ''
            },
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': 'test-login-user-with-email-failed',
                'password': 'invalid-password'
            }
        )
        assert login_admin_response.status_code == 404



from config import (
    app_enable_rpc_server, app_enable_event_handler, app_enable_api,
    app_enable_snake_zrb_module_name_module
)
from component.log import logger
from component.app import app
from component.messagebus import consumer, publisher
from component.rpc import rpc_caller, rpc_server
from module.auth.component import authorizer
from module.snake_zrb_module_name.api import register_api
from module.snake_zrb_module_name.event import register_event
from module.snake_zrb_module_name.rpc import register_rpc


def register_snake_zrb_module_name():
    if not app_enable_snake_zrb_module_name_module:
        logger.info('🥪 Skip registering "snake_zrb_module_name"')
        return
    if app_enable_api:
        register_api(
            logger=logger,
            app=app,
            authorizer=authorizer,
            rpc_caller=rpc_caller,
            publisher=publisher
        )
    if app_enable_event_handler:
        register_event(
            logger=logger,
            consumer=consumer,
            rpc_caller=rpc_caller,
            publisher=publisher
        )
    if app_enable_rpc_server:
        register_rpc(
            logger=logger,
            rpc_server=rpc_server,
            rpc_caller=rpc_caller,
            publisher=publisher
        )



from zrb.helper.typecheck import typechecked
import libcst as cst


@typechecked
class TaskTransformer(cst.CSTTransformer):
    def __init__(self, task_name: str, upstream_task_name: str):
        super().__init__()
        self.task_name: str = task_name
        self.upstream_task_name: str = upstream_task_name
        self._on_task_call = False
        self._upstream_added = False

    def _is_task(self, node: cst.Call) -> bool:
        if (
            not isinstance(node.func, cst.Name)
            or node.func.value != "Task"
        ):
            return False
        # Look for name keyword
        for name_arg in node.args:
            if (
                isinstance(name_arg, cst.Arg)
                and name_arg.keyword.value == "name"
                and name_arg.value.evaluated_value == self.task_name
            ):
                return True
        return False

    def visit_Call(self, node: cst.Call):
        # Visit all function call (or class initiation)
        # If it is the task we seek, flag it
        if self._is_task(node):
            self._on_task_call = True

    def leave_Call(
        self, original_node: cst.Call, updated_node: cst.Call
    ) -> cst.Call:
        # Remove flags.
        # If upstreams argument not found (see: self.leave_Arg), add it
        if not self._is_task(updated_node):
            return updated_node
        self._on_task_call = False
        if self._upstream_added:
            return updated_node
        new_args = [arg for arg in updated_node.args]
        new_args.append(cst.Arg(
            keyword=cst.parse_expression('upstreams'),
            value=cst.List([
                cst.Element(cst.parse_expression(self.upstream_task_name))
            ])
        ))
        self._upstream_added = True
        return updated_node.with_changes(
            args=new_args
        )

    def leave_Arg(self, orginnal_node: cst.Arg, updated_node: cst.Arg):
        # Once the flag was set (see: self.visit_Call), look for
        # `upstreams` argument and modify the value
        if (
            not self._on_task_call
            or updated_node.keyword is None
            or updated_node.keyword.value != 'upstreams'
        ):
            return updated_node
        new_elements = [
            old_element for old_element in updated_node.value.elements
        ]
        new_elements.append(cst.Element(
            value=cst.parse_expression(self.upstream_task_name)
        ))
        self._upstream_added = True
        return updated_node.with_changes(
            value=cst.List(new_elements)
        )


@typechecked
def add_upstream_to_task(
    code: str, task_name: str, upstream_task_name: str
) -> str:
    """
    Transformer to add an upstream task to a Task object.

    Args:
        code (str): The code to modify.
        task_name (str): Name of the task to modify.
        upstream_task_name (str): Name of the upstream task to add.

    Returns:
        A modified version of the input code with the upstream task added.
    """
    transformer: TaskTransformer = TaskTransformer(
        task_name, upstream_task_name
    )
    original_tree: cst.Module = cst.parse_module(code)
    modified_tree: cst.Module = original_tree.visit(transformer)
    return modified_tree.code



from zrb.helper.typing import List
from zrb.task.decorator import python_task
from zrb.task.any_task import AnyTask


def create_on_triggered(logs: List[str]):
    def on_triggered(task: AnyTask):
        logs.append('triggered')
    return on_triggered


def create_on_waiting(logs: List[str]):
    def on_waiting(task: AnyTask):
        logs.append('waiting')
    return on_waiting


def create_on_skipped(logs: List[str]):
    def on_skipped(task: AnyTask):
        logs.append('skipped')
    return on_skipped


def create_on_started(logs: List[str]):
    def on_started(task: AnyTask):
        logs.append('started')
    return on_started


def create_on_ready(logs: List[str]):
    def on_ready(task: AnyTask):
        logs.append('ready')
    return on_ready


def create_on_retry(logs: List[str]):
    def on_retry(task: AnyTask):
        logs.append('retry')
    return on_retry


def create_on_failed(logs: List[str]):
    def on_failed(task: AnyTask, is_last_attempt: bool, exception: Exception):
        if is_last_attempt:
            logs.append('failed for good')
            return
        logs.append('failed')
    return on_failed


def test_success_task():
    logs = []

    @python_task(
        name='task',
        on_triggered=create_on_triggered(logs),
        on_waiting=create_on_waiting(logs),
        on_skipped=create_on_skipped(logs),
        on_started=create_on_started(logs),
        on_ready=create_on_ready(logs),
        on_retry=create_on_retry(logs),
        on_failed=create_on_failed(logs)
    )
    def task(*args, **kwargs) -> str:
        return 'ok'

    function = task.to_function()
    function()
    assert len(logs) == 4
    assert logs[0] == 'triggered'
    assert logs[1] == 'waiting'
    assert logs[2] == 'started'
    assert logs[3] == 'ready'


def test_skipped_task():
    logs = []

    @python_task(
        name='task',
        should_execute=False,
        on_triggered=create_on_triggered(logs),
        on_waiting=create_on_waiting(logs),
        on_skipped=create_on_skipped(logs),
        on_started=create_on_started(logs),
        on_ready=create_on_ready(logs),
        on_retry=create_on_retry(logs),
        on_failed=create_on_failed(logs)
    )
    def task(*args, **kwargs) -> str:
        return 'ok'

    function = task.to_function()
    function()
    assert len(logs) == 4
    assert logs[0] == 'triggered'
    assert logs[1] == 'waiting'
    assert logs[2] == 'skipped'
    assert logs[3] == 'ready'


def test_failed_for_good_task():
    logs = []

    @python_task(
        name='task',
        retry=2,
        on_triggered=create_on_triggered(logs),
        on_waiting=create_on_waiting(logs),
        on_skipped=create_on_skipped(logs),
        on_started=create_on_started(logs),
        on_ready=create_on_ready(logs),
        on_retry=create_on_retry(logs),
        on_failed=create_on_failed(logs)
    )
    def task(*args, **kwargs) -> str:
        raise Exception('error')

    function = task.to_function()
    is_error = False
    try:
        function()
    except Exception:
        is_error = True
    assert is_error
    assert len(logs) == 10
    assert logs[0] == 'triggered'
    assert logs[1] == 'waiting'
    assert logs[2] == 'started'
    assert logs[3] == 'failed'
    assert logs[4] == 'retry'
    assert logs[5] == 'started'
    assert logs[6] == 'failed'
    assert logs[7] == 'retry'
    assert logs[8] == 'started'
    assert logs[9] == 'failed for good'


def test_failed_then_success_task():
    logs = []
    state = {'retry': 0}

    @python_task(
        name='task',
        on_triggered=create_on_triggered(logs),
        on_waiting=create_on_waiting(logs),
        on_skipped=create_on_skipped(logs),
        on_started=create_on_started(logs),
        on_ready=create_on_ready(logs),
        on_retry=create_on_retry(logs),
        on_failed=create_on_failed(logs)
    )
    def task(*args, **kwargs) -> str:
        state['retry'] += 1
        if state['retry'] == 3:
            return 'ok'
        raise Exception('Try again')

    function = task.to_function()
    function()
    assert len(logs) == 10
    assert logs[0] == 'triggered'
    assert logs[1] == 'waiting'
    assert logs[2] == 'started'
    assert logs[3] == 'failed'
    assert logs[4] == 'retry'
    assert logs[5] == 'started'
    assert logs[6] == 'failed'
    assert logs[7] == 'retry'
    assert logs[8] == 'started'
    assert logs[9] == 'ready'



from zrb.helper.string.conversion import to_boolean, to_logging_level
from zrb.helper.typecheck import typechecked
import os
import importlib.metadata as metadata


@typechecked
def get_version() -> str:
    try:
        return metadata.version("zrb")
    except metadata.PackageNotFoundError:
        import flit
        meta = flit.read_module_metadata("zrb")
        return str(meta["module"]["version"])


@typechecked
def get_current_shell() -> str:
    current_shell = os.getenv('SHELL', '')
    if current_shell.endswith('zsh'):
        return 'zsh'
    return 'bash'


default_shell = os.getenv('ZRB_SHELL', get_current_shell())
init_script_str = os.getenv('ZRB_INIT_SCRIPTS', '')
init_scripts = init_script_str.split(':') if init_script_str != '' else []
logging_level = to_logging_level(os.getenv('ZRB_LOGGING_LEVEL', 'WARNING'))
should_load_builtin = to_boolean(os.getenv('ZRB_SHOULD_LOAD_BUILTIN', '1'))
env_prefix = os.getenv('ZRB_ENV', '')
show_advertisement = to_boolean(os.getenv('ZRB_SHOW_ADVERTISEMENT', '1'))
show_prompt = to_boolean(os.getenv('ZRB_SHOW_PROMPT', '1'))
show_time = to_boolean(os.getenv('ZRB_SHOW_TIME', '1'))
version = get_version()



from zrb.helper.typing import Any, Callable, Iterable, Optional, Union
from zrb.helper.typecheck import typechecked
from zrb.task.any_task import AnyTask
from zrb.task.any_task_event_handler import (
    OnTriggered, OnWaiting, OnSkipped, OnStarted, OnReady, OnRetry, OnFailed
)
from zrb.task_env.env import Env
from zrb.task_env.env_file import EnvFile
from zrb.task_group.group import Group
from zrb.task_input.any_input import AnyInput
from zrb.task.cmd_task import CmdVal, CmdTask
from zrb.task.base_remote_cmd_task import RemoteConfig, BaseRemoteCmdTask

import os
import pathlib


CURRENT_DIR = os.path.dirname(__file__)
SHELL_SCRIPT_DIR = os.path.join(CURRENT_DIR, '..', 'shell-scripts')
with open(os.path.join(SHELL_SCRIPT_DIR, 'ssh-util.sh')) as file:
    SSH_UTIL_SCRIPT = file.read()

ensure_ssh_is_installed = CmdTask(
    name='ensure-ssh-is-installed',
    cmd_path=[
        os.path.join(SHELL_SCRIPT_DIR, '_common-util.sh'),
        os.path.join(SHELL_SCRIPT_DIR, 'ensure-ssh-is-installed.sh')
    ],
    preexec_fn=None
)


@typechecked
class RemoteCmdTask(BaseRemoteCmdTask):
    def __init__(
        self,
        name: str,
        remote_configs: Iterable[RemoteConfig],
        group: Optional[Group] = None,
        inputs: Iterable[AnyInput] = [],
        envs: Iterable[Env] = [],
        env_files: Iterable[EnvFile] = [],
        icon: Optional[str] = None,
        color: Optional[str] = None,
        description: str = '',
        executable: Optional[str] = None,
        cmd: CmdVal = '',
        cmd_path: CmdVal = '',
        cwd: Optional[Union[str, pathlib.Path]] = None,
        upstreams: Iterable[AnyTask] = [],
        on_triggered: Optional[OnTriggered] = None,
        on_waiting: Optional[OnWaiting] = None,
        on_skipped: Optional[OnSkipped] = None,
        on_started: Optional[OnStarted] = None,
        on_ready: Optional[OnReady] = None,
        on_retry: Optional[OnRetry] = None,
        on_failed: Optional[OnFailed] = None,
        checkers: Iterable[AnyTask] = [],
        checking_interval: Union[float, int] = 0,
        retry: int = 2,
        retry_interval: Union[float, int] = 1,
        max_output_line: int = 1000,
        max_error_line: int = 1000,
        preexec_fn: Optional[Callable[[], Any]] = os.setsid,
        should_execute: Union[bool, str, Callable[..., bool]] = True
    ):
        pre_cmd = '\n'.join([
            SSH_UTIL_SCRIPT,
            '_SCRIPT="$(cat <<\'ENDSCRIPT\'',
        ])
        post_cmd = '\n'.join([
            'ENDSCRIPT',
            ')"',
            'auth_ssh "$_SCRIPT"'
        ])
        BaseRemoteCmdTask.__init__(
            self,
            name=name,
            remote_configs=remote_configs,
            group=group,
            inputs=inputs,
            envs=envs,
            env_files=env_files,
            icon=icon,
            color=color,
            description=description,
            executable=executable,
            pre_cmd=pre_cmd,
            cmd=cmd,
            cmd_path=cmd_path,
            post_cmd=post_cmd,
            cwd=cwd,
            upstreams=[ensure_ssh_is_installed] + upstreams,
            on_triggered=on_triggered,
            on_waiting=on_waiting,
            on_skipped=on_skipped,
            on_started=on_started,
            on_ready=on_ready,
            on_retry=on_retry,
            on_failed=on_failed,
            checkers=checkers,
            checking_interval=checking_interval,
            retry=retry,
            retry_interval=retry_interval,
            max_output_line=max_output_line,
            max_error_line=max_error_line,
            preexec_fn=preexec_fn,
            should_execute=should_execute
        )

    def __repr__(self) -> str:
        return f'<RemoteCmdTask name={self._name}>'



from core.rpc.rpc import Caller, Server
from core.rpc.messagebus.caller import MessagebusCaller
from core.rpc.messagebus.server import MessagebusServer

assert Caller
assert Server
assert MessagebusCaller
assert MessagebusServer



from zrb import CmdTask, EnvFile, BoolInput, StrInput, IntInput, runner
from zrb.builtin.group import project_group
from ._common import CURRENT_DIR, RESOURCE_DIR
import os

###############################################################################
# Constants
###############################################################################

LOAD_TEST_DIR = os.path.join(RESOURCE_DIR, 'loadtest')
LOAD_TEST_TEMPLATE_ENV_FILE_NAME = os.path.join(RESOURCE_DIR, 'template.env')

###############################################################################
# Input Definitions
###############################################################################

headless_input = BoolInput(
    name='kebab-zrb-app-name-load-test-headless',
    default=True,
    description='Load test UI headless',
    prompt='Load test UI headless (if True, there will be no UI)',
)

web_port_input = IntInput(
    name='kebab-zrb-app-name-load-test-port',
    default=8089,
    description='Load test UI web port',
    prompt='Load test UI web port (Only works if headless is False)',
)

users_input = IntInput(
    name='kebab-zrb-app-name-load-test-users',
    default=200,
    description='Load test users',
    prompt='Load test users',
)

spawn_rate_input = IntInput(
    name='kebab-zrb-app-name-load-test-spawn-rate',
    default=10,
    description='Load test spawn rate',
    prompt='Load test spawn rate',
)

url_input = StrInput(
    name='kebab-zrb-app-name-load-test-url',
    default='http://localhost:zrbAppHttpPort',
    description='Load test url',
    prompt='Load test url',
)

###############################################################################
# Env file Definitions
###############################################################################

load_test_env_file = EnvFile(
    env_file=LOAD_TEST_TEMPLATE_ENV_FILE_NAME,
    prefix='LOAD_TEST_ZRB_ENV_PREFIX'
)

###############################################################################
# Task Definitions
###############################################################################

prepare_snake_zrb_app_name_load_test = CmdTask(
    icon='🚤',
    name='prepare-kebab-zrb-app-name-load-test',
    description='Prepare load test for human readable zrb app name',
    group=project_group,
    cwd=LOAD_TEST_DIR,
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'app-prepare-load-test.sh'),
    ]
)
runner.register(prepare_snake_zrb_app_name_load_test)

load_test_snake_zrb_app_name_load_test = CmdTask(
    icon='🧪',
    name='load-test-kebab-zrb-app-name',
    description='Load test human readable zrb app name',
    group=project_group,
    upstreams=[prepare_snake_zrb_app_name_load_test],
    inputs=[
        headless_input,
        web_port_input,
        users_input,
        spawn_rate_input,
        url_input,
    ],
    cwd=LOAD_TEST_DIR,
    env_files=[load_test_env_file],
    cmd_path=[
        os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
        os.path.join(CURRENT_DIR, 'cmd', 'app-load-test.sh'),
    ]
)
runner.register(load_test_snake_zrb_app_name_load_test)



from typing import List
from zrb.task.cmd_task import CmdTask
from zrb.task.task import Task
from zrb.task.flow_task import FlowTask
from zrb.task.decorator import python_task


def test_flow_task():
    flow_task = FlowTask(
        name='flow_task',
        steps=[
            [
                CmdTask(name='create-sodium', cmd='echo "Na"'),
                CmdTask(name='create-chlorine', cmd='echo "Cl"'),
            ],
            Task(
                name='create-salt',
                run=lambda *args, **kwargs: "NaCl"
            ),
            FlowTask(
                name='create-saline-water',
                steps=[
                    Task(
                        name='create-water',
                        run=lambda *args, **kwargs: "H2O + NaCl"
                    )
                ]
            )
        ]
    )
    function = flow_task.to_function()
    is_error: bool = False
    try:
        function()
    except Exception:
        is_error = True
    assert not is_error


def test_flow_task_with_existing_tasks():
    outputs: List[str] = []

    @python_task(
        name='prepare-lab'
    )
    def prepare_lab(*args, **kwargs):
        outputs.append('Lab prepared')

    @python_task(
        name='create-sodium'
    )
    def create_sodium(*args, **kwargs):
        outputs.append('Na')

    @python_task(
        name='create-chlorine'
    )
    def create_chlorine(*args, **kwargs):
        outputs.append('Cl')

    @python_task(
        name='create-salt'
    )
    def create_salt(*args, **kwargs):
        outputs.append('NaCl')

    flow_task = FlowTask(
        name='flow_task',
        upstreams=[prepare_lab],
        steps=[
            [
                create_sodium,
                create_chlorine
            ],
            create_salt
        ]
    )
    function = flow_task.to_function()
    function()
    assert outputs[0] == 'Lab prepared'
    assert 'Na' in outputs
    assert 'Cl' in outputs
    assert outputs[3] == 'NaCl'




from zrb.builtin.generator.project.create import create_project
from zrb.builtin.generator.cmd_task.add import add_cmd_task
from zrb.builtin.generator.docker_compose_task.add import (
    add_docker_compose_task
)
from zrb.builtin.generator.python_task.add import add_python_task
from zrb.builtin.generator.simple_python_app import (
    add as add_simple_python_app
)
from zrb.builtin.generator.fastapp import add as add_fastapp
from zrb.builtin.generator.fastapp_module import add as add_fastapp_module
from zrb.builtin.generator.fastapp_crud import add as add_fastapp_crud
from zrb.builtin.generator.fastapp_field import add as add_fastapp_field
from zrb.builtin.generator.pip_package import add as add_pip_package
from zrb.builtin.generator.app_generator import add as add_app_generator

assert create_project
assert add_cmd_task
assert add_docker_compose_task
assert add_python_task
assert add_simple_python_app
assert add_fastapp
assert add_fastapp_module
assert add_fastapp_crud
assert add_fastapp_field
assert add_pip_package
assert add_app_generator



from zrb import HTTPChecker

snake_zrb_app_name_checker = HTTPChecker(
    name='check-kebab-zrb-app-name',
    is_https='{{input.snake_zrb_app_name_https}}',
    host='{{input.snake_zrb_app_name_host}}',
    port='{{env.APP_PORT}}',
    url='/'
)

snake_zrb_app_name_container_checker = HTTPChecker(
    name='check-kebab-zrb-app-name',
    is_https='{{input.snake_zrb_app_name_https}}',
    host='{{input.snake_zrb_app_name_host}}',
    port='{{env.HOST_PORT}}',
    url='/'
)



from zrb.helper.typing import Any
from zrb.builtin.group import git_group
from zrb.task.decorator import python_task
from zrb.task.task import Task
from zrb.task_input.str_input import StrInput
from zrb.task_input.bool_input import BoolInput
from zrb.runner import runner
from zrb.helper.git.detect_changes import get_modified_file_states
from zrb.helper.accessories.color import colored
from zrb.helper.python_task import show_lines

###############################################################################
# Task Definitions
###############################################################################


@python_task(
    name='get-file-changes',
    group=git_group,
    description='Get modified files',
    inputs=[
        StrInput(
            name='commit',
            shortcut='c',
            description='commit hash/tag',
            prompt='Commit hash/Tag',
            default='HEAD'
        ),
       BoolInput(
            name='include-new',
            description='include new files',
            prompt='Include new files',
            default=True
        ),
        BoolInput(
            name='include-removed',
            description='include removed file',
            prompt='Include removed file',
            default=True
        ), 
        BoolInput(
            name='include-updated',
            description='include updated file',
            prompt='Include removed file',
            default=True
        ),
    ],
    runner=runner
)
async def get_file_changes(*args: Any, **kwargs: Any):
    commit = kwargs.get('commit', 'HEAD')
    include_new = kwargs.get('include_new', True)
    include_removed = kwargs.get('include_removed', True)
    include_updated = kwargs.get('include_updated', True)
    task: Task = kwargs['_task']
    modified_file_states = get_modified_file_states(commit)
    modified_file_keys = []
    output = []
    for modified_file, state in modified_file_states.items():
        if include_updated and state.minus and state.plus:
            output.append(colored(f'+- {modified_file}', color='yellow'))
            modified_file_keys.append(modified_file)
            continue
        if include_removed and state.minus and not state.plus:
            output.append(colored(f'-- {modified_file}', color='red'))
            modified_file_keys.append(modified_file)
            continue
        if include_new and state.plus and not state.minus:
            output.append(colored(f'++ {modified_file}', color='green'))
            modified_file_keys.append(modified_file)
            continue
    show_lines(kwargs['_task'], *output)
    modified_file_keys.sort()
    return '\n'.join(modified_file_keys)



from zrb.builtin.group import ubuntu_group, ubuntu_install_group
from zrb.task.cmd_task import CmdTask
from zrb.runner import runner

###############################################################################
# Task Definitions
###############################################################################


update_task = CmdTask(
    name='update',
    group=ubuntu_group,
    description='Update ubuntu',
    cmd=[
        'sudo apt update',
        'sudo apt upgrade -y',
    ],
    checking_interval=3,
    preexec_fn=None
)
runner.register(update_task)

install_toys = CmdTask(
    name='toys',
    group=ubuntu_install_group,
    description='Install ubuntu toy packages',
    cmd=[
        'sudo apt install -y lolcat cowsay figlet neofetch',
    ],
    upstreams=[update_task],
    checking_interval=3,
    preexec_fn=None
)
runner.register(install_toys)

install_packages = CmdTask(
    name='packages',
    group=ubuntu_install_group,
    description='Install essential ubuntu packages',
    cmd=[
        'sudo apt install -y \\',
        'build-essential python3-distutils libssl-dev zlib1g-dev \\'
        'libbz2-dev libreadline-dev libsqlite3-dev libpq-dev python3-dev \\',
        'llvm libncurses5-dev libncursesw5-dev xz-utils tk-dev libffi-dev \\',
        'liblzma-dev python3-openssl libblas-dev liblapack-dev rustc \\',
        'golang gfortran fd-find ripgrep wget curl git ncat zip unzip \\',
        'cmake make tree tmux zsh neovim xdotool xsel'
    ],
    upstreams=[update_task],
    checking_interval=3,
    preexec_fn=None
)
runner.register(install_packages)



from zrb import DockerComposeTask, Env, StrInput, runner
from zrb.builtin.group import project_group
from ._common import (
    RESOURCE_DIR, local_input
)

###############################################################################
# Input Definitions
###############################################################################

image_input = StrInput(
    name='kebab-zrb-app-name-image',
    description='Image name of "kebab-zrb-app-name"',
    prompt='Image name of "kebab-zrb-app-name"',
    default='zrb-app-image-name:latest'
)


###############################################################################
# Env Definitions
###############################################################################

image_env = Env(
    name='IMAGE',
    os_name='CONTAINER_ZRB_ENV_PREFIX_IMAGE',
    default='{{input.snake_zrb_app_name_image}}'
)

###############################################################################
# Task Definitions
###############################################################################

build_snake_zrb_app_name_image = DockerComposeTask(
    icon='🏭',
    name='build-kebab-zrb-app-name-image',
    description='Build human readable zrb app name image',
    group=project_group,
    inputs=[
        local_input,
        image_input,
    ],
    envs=[image_env],
    should_execute='{{ input.local_snake_zrb_app_name}}',
    cwd=RESOURCE_DIR,
    compose_cmd='build',
    compose_flags=[
        '--no-cache'
    ],
    compose_env_prefix='CONTAINER_ZRB_ENV_PREFIX',
)
runner.register(build_snake_zrb_app_name_image)

push_snake_zrb_app_name_image = DockerComposeTask(
    icon='📰',
    name='push-kebab-zrb-app-name-image',
    description='Push human readable zrb app name image',
    group=project_group,
    inputs=[
        local_input,
        image_input,
    ],
    envs=[image_env],
    upstreams=[build_snake_zrb_app_name_image],
    cwd=RESOURCE_DIR,
    compose_cmd='push',
    compose_env_prefix='CONTAINER_ZRB_ENV_PREFIX',
)
runner.register(push_snake_zrb_app_name_image)



from zrb import DockerComposeTask, HTTPChecker, Env, EnvFile, runner
from zrb.builtin.group import project_group
import os

###############################################################################
# Constants
###############################################################################

CURRENT_DIR = os.path.dirname(__file__)
PROJECT_DIR = os.path.abspath(os.path.join(CURRENT_DIR, '..'))
RESOURCE_DIR = os.path.join(PROJECT_DIR, 'src', 'kebab-zrb-task-name')

###############################################################################
# Task Definitions
###############################################################################

snake_zrb_task_name = DockerComposeTask(
    name='kebab-zrb-task-name',
    description='human readable zrb task name',
    group=project_group,
    cwd=RESOURCE_DIR,
    compose_cmd='zrbComposeCommand',
    compose_env_prefix='ZRB_ENV_PREFIX',
    env_files=[
        EnvFile(
            env_file=os.path.join(RESOURCE_DIR, 'docker-compose.env'),
            prefix='ZRB_ENV_PREFIX'
        )
    ],
    envs=[
        Env(
            name='HOST_PORT',
            os_name='ZRB_ENV_PREFIX_HOST_PORT',
            default='zrbHttpPort'
        ),
    ],
    checkers=[
        HTTPChecker(
            name='check-kebab-zrb-task-name',
            port='{{env.HOST_PORT}}'
        )
    ]
)
runner.register(snake_zrb_task_name)



from zrb.task.base_task import BaseTask
from zrb.helper.typecheck import typechecked


@typechecked
class Task(BaseTask):
    '''
    Alias for BaseTask
    '''

    def __repr__(self) -> str:
        return f'<Task name={self._name}>'



from zrb.helper.typecheck import typechecked
import libcst as cst


@typechecked
class AddArgumentTransformer(cst.CSTTransformer):
    def __init__(self, function_name: str, argument_name: str):
        self.function_name = function_name
        self.argument_name = argument_name
        super().__init__()

    def leave_Call(
        self, original_node: cst.Call, updated_node: cst.Call
    ) -> cst.Call:
        if (
            isinstance(updated_node.func, cst.Name)
            and updated_node.func.value == self.function_name
        ):
            new_args = updated_node.args + (
                cst.Arg(cst.Name(self.argument_name)),
            )
            return updated_node.with_changes(args=new_args)
        return updated_node


@typechecked
def add_argument_to_function_call(
    code: str, function_name: str, argument_name: str
) -> str:
    module = cst.parse_module(code)
    transformed_module = module.visit(
        AddArgumentTransformer(function_name, argument_name)
    )
    return transformed_module.code



from typing import AsyncIterator
from httpx import AsyncClient
from src.config import app_auth_admin_username, app_auth_admin_password
import pytest

inserted_success_data = {
    'name': 'test-create-group-success',
    'description': '',
    'permissions': [],
}
to_be_updated_success_data = {
    'name': 'test-group-to-be-updated-success',
    'description': '',
    'permissions': [],
}
updated_success_data = {
    'name': 'test-group-updated-success',
    'description': 'new-description',
    'permissions': [],
}
to_be_deleted_success_data = {
    'name': 'test-group-to-be-deleted-success',
    'description': '',
    'permissions': [],
}


@pytest.mark.asyncio
async def test_insert_group_and_get_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )

        # create group
        create_response = await client.post(
            '/api/v1/auth/groups',
            json=inserted_success_data,
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        json_create_response = create_response.json()
        create_response_id = json_create_response.get('id', '')
        assert create_response_id != ''

        # get_by_id
        get_by_id_response = await client.get(
            f'/api/v1/auth/groups/{create_response_id}',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_by_id_response.status_code == 200
        json_get_by_id_response = get_by_id_response.json()
        get_by_id_response_id = json_get_by_id_response.get('id', '')
        assert get_by_id_response_id == create_response_id

        # get
        get_response = await client.get(
            '/api/v1/auth/groups',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_response.status_code == 200
        json_get_response = get_response.json()
        get_response_count = json_get_response.get('count', '')
        assert get_response_count > 0
        get_response_data = json_get_response.get('data', [])
        get_created_response_data = [
            row for row in get_response_data
            if row['id'] == create_response_id
        ]
        assert len(get_created_response_data) == 1


@pytest.mark.asyncio
async def test_update_group_and_get_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password,
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )

        # create group
        create_response = await client.post(
            '/api/v1/auth/groups',
            json=to_be_updated_success_data,
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        json_create_response = create_response.json()
        create_response_id = json_create_response.get('id', '')
        assert create_response_id != ''

        # update group
        update_response = await client.put(
            f'/api/v1/auth/groups/{create_response_id}',
            json=updated_success_data,
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert update_response.status_code == 200
        json_update_response = update_response.json()
        update_response_id = json_update_response.get('id', '')
        assert update_response_id == create_response_id

        # get_by_id
        get_by_id_response = await client.get(
            f'/api/v1/auth/groups/{create_response_id}',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_by_id_response.status_code == 200
        json_get_by_id_response = get_by_id_response.json()
        get_by_id_response_id = json_get_by_id_response.get('id', '')
        assert get_by_id_response_id == create_response_id
        for key, expected_value in updated_success_data.items():
            actual_value = json_get_by_id_response.get(key, '')
            assert f'{key}:{actual_value}' == f'{key}:{expected_value}'

        # get
        get_response = await client.get(
            '/api/v1/auth/groups',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_response.status_code == 200
        json_get_response = get_response.json()
        get_response_count = json_get_response.get('count', '')
        assert get_response_count > 0
        get_response_data = json_get_response.get('data', [])
        get_created_response_data = [
            row for row in get_response_data
            if row['id'] == create_response_id
        ]
        assert len(get_created_response_data) == 1


@pytest.mark.asyncio
async def test_delete_group_and_get_success(
    test_client_generator: AsyncIterator[AsyncClient]
):
    async for client in test_client_generator:
        # login
        login_admin_response = await client.post(
            '/api/v1/auth/login',
            json={
                'identity': app_auth_admin_username,
                'password': app_auth_admin_password
            }
        )
        assert login_admin_response.status_code == 200
        admin_access_token = login_admin_response.json().get(
            'access_token', ''
        )

        # create group
        create_response = await client.post(
            '/api/v1/auth/groups',
            json=to_be_deleted_success_data,
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert create_response.status_code == 200
        json_create_response = create_response.json()
        create_response_id = json_create_response.get('id', '')
        assert create_response_id != ''

        # create group
        delete_response = await client.delete(
            f'/api/v1/auth/groups/{create_response_id}',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert delete_response.status_code == 200
        json_delete_response = delete_response.json()
        delete_response_id = json_delete_response.get('id', '')
        assert delete_response_id == create_response_id

        # get_by_id
        get_by_id_response = await client.get(
            f'/api/v1/auth/groups/{create_response_id}',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_by_id_response.status_code == 404

        # get
        get_response = await client.get(
            '/api/v1/auth/groups',
            headers={
                'Authorization': 'Bearer ' + admin_access_token
            }
        )
        assert get_response.status_code == 200
        json_get_response = get_response.json()
        get_response_count = json_get_response.get('count', '')
        assert get_response_count > 0
        get_response_data = json_get_response.get('data', [])
        get_created_response_data = [
            row for row in get_response_data
            if row['id'] == create_response_id
        ]
        assert len(get_created_response_data) == 0



from typing import List
from zrb import CmdTask, Env, HTTPChecker, Task, EnvFile
from zrb.helper.util import to_snake_case, to_kebab_case
from ._common import (
    CURRENT_DIR, APP_DIR, should_start_local_microservices,
    APP_TEMPLATE_ENV_FILE_NAME, MODULES,
    local_input, run_mode_input, host_input, https_input,
    enable_monitoring_input, app_enable_otel_env
)
import os


def get_start_microservices(upstreams: List[Task]) -> List[Task]:
    disable_all_module_envs = _get_disable_all_module_envs()
    start_microservices: List[Task] = []
    for module_index, module_name in enumerate(MODULES):
        kebab_module_name = to_kebab_case(module_name)
        # Define start service task
        start_service = CmdTask(
            name=f'start-kebab-zrb-app-name-{kebab_module_name}-service',
            inputs=[
                local_input,
                run_mode_input,
                host_input,
                https_input,
                enable_monitoring_input,
            ],
            should_execute=should_start_local_microservices,
            upstreams=upstreams,
            cwd=APP_DIR,
            env_files=[_get_service_env_file(module_name)],
            envs=disable_all_module_envs + _get_service_envs(
                zrbAppHttpPort, module_index, module_name
            ),
            cmd_path=[
                os.path.join(CURRENT_DIR, 'cmd', 'activate-venv.sh'),
                os.path.join(CURRENT_DIR, 'cmd', 'app-start.sh'),
            ],
            checkers=[
                HTTPChecker(
                    name=f'check-kebab-zrb-app-name-{kebab_module_name}-service',
                    host='{{input.snake_zrb_app_name_host}}',
                    url='/readiness',
                    port='{{env.APP_PORT}}',
                    is_https='{{input.snake_zrb_app_name_https}}',
                    should_execute=should_start_local_microservices
                )
            ]
        )
        start_microservices.append(start_service)
    return start_microservices


def _get_service_envs(
    base_port: int, module_index: int, module_name: str
) -> List[Env]:
    kebab_module_name = to_kebab_case(module_name)
    snake_module_name = to_snake_case(module_name)
    upper_snake_module_name = snake_module_name.upper()
    # Define service env
    app_broker_type_env = Env(
        name='APP_BROKER_TYPE',
        os_name=f'ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_BROKER_TYPE',
        default='rabbitmq'
    )
    app_port_env = Env(
        name='APP_PORT',
        os_name=f'ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_PORT',
        default=str(base_port + module_index + 1)
    )
    enable_module_env = Env(
        name=f'APP_ENABLE_{upper_snake_module_name}_MODULE',
        os_name='',
        default='true'
    )
    zrb_app_name_env = Env(
        name='APP_NAME',
        os_name=f'ZRB_ENV_PREFIX_{upper_snake_module_name}_APP_NAME',
        default=f'kebab-zrb-app-name-{kebab_module_name}-service'
    )
    return [
        app_broker_type_env,
        app_enable_otel_env,
        app_port_env,
        zrb_app_name_env,
        enable_module_env,
    ]


def _get_service_env_file(module_name: str) -> EnvFile:
    snake_module_name = to_snake_case(module_name)
    upper_snake_module_name = snake_module_name.upper()
    return EnvFile(
        env_file=APP_TEMPLATE_ENV_FILE_NAME,
        prefix=f'ZRB_ENV_PREFIX_{upper_snake_module_name}'
    )


def _get_disable_all_module_envs() -> List[Env]:
    disable_all_module_envs: List[Env] = []
    for module in MODULES:
        snake_module_name = to_snake_case(module)
        upper_snake_module_name = snake_module_name.upper()
        disable_all_module_envs.append(Env(
            name=f'APP_ENABLE_{upper_snake_module_name}_MODULE',
            os_name='',
            default='false'
        ))
    return disable_all_module_envs



from fastapi import FastAPI, Depends
from logging import Logger
from core.messagebus import Publisher
from core.rpc import Caller
from core.error import HTTPAPIException
from module.auth.core import Authorizer
from module.log.schema.activity import Activity, ActivityResult
from module.auth.schema.token import AccessTokenData
from module.auth.component import access_token_scheme


def register_api(
    logger: Logger,
    app: FastAPI,
    authorizer: Authorizer,
    rpc_caller: Caller,
    publisher: Publisher
):
    logger.info('🥪 Register API for "log.activity"')

    @app.get(
        '/api/v1/log/activities', response_model=ActivityResult
    )
    async def get_activitys(
        keyword: str = '', limit: int = 100, offset: int = 0,
        user_token_data: AccessTokenData = Depends(access_token_scheme)
    ):
        if not await authorizer.is_having_permission(
            user_token_data.user_id, 'log:activity:get'
        ):
            raise HTTPAPIException(403, 'Unauthorized')
        try:
            result_dict = await rpc_caller.call(
                'log_get_activity',
                keyword=keyword,
                criterion={},
                limit=limit,
                offset=offset,
                user_token_data=user_token_data.dict()
            )
            return ActivityResult(**result_dict)
        except Exception as e:
            raise HTTPAPIException(error=e)

    @app.get(
        '/api/v1/log/activities/{id}', response_model=Activity
    )
    async def get_activity_by_id(
        id: str,
        user_token_data: AccessTokenData = Depends(access_token_scheme)
    ):
        if not await authorizer.is_having_permission(
            user_token_data.user_id, 'log:activity:get_by_id'
        ):
            raise HTTPAPIException(403, 'Unauthorized')
        try:
            result_dict = await rpc_caller.call(
                'log_get_activity_by_id',
                id=id, user_token_data=user_token_data.dict()
            )
            return Activity(**result_dict)
        except Exception as e:
            raise HTTPAPIException(error=e)



from module.auth.core.password_hasher.password_hasher import PasswordHasher
import bcrypt


class BcryptPasswordHasher(PasswordHasher):

    def hash_password(self, password: str) -> str:
        return bcrypt.hashpw(
            password.encode('utf-8'), bcrypt.gensalt()
        ).decode('utf-8')

    def check_password(self, password: str, hashed_password: str) -> bool:
        return bcrypt.checkpw(
            password.encode('utf-8'), hashed_password.encode('utf-8')
        )



from zrb.task.cmd_task import CmdTask
from zrb.builtin.group import process_group
from zrb.task_input.int_input import IntInput
from zrb.task_input.str_input import StrInput
from zrb.runner import runner

###############################################################################
# Task Definitions
###############################################################################

get_pid_by_port = CmdTask(
    name='get-pid-by-port',
    group=process_group,
    description='Get PID by port',
    inputs=[
        IntInput(
            name='port',
            shortcut='p',
            default='8080',
            description='Port to be checked',
            prompt='Port to be checked',
        )
    ],
    cmd="lsof -i :{{ input.port }} -n -P | awk 'NR>1 {print $2}'",
    checking_interval=3
)
runner.register(get_pid_by_port)

get_pid_by_name = CmdTask(
    name='get-pid-by-name',
    group=process_group,
    description='Get PID by name',
    inputs=[
        StrInput(
            name='name',
            shortcut='n',
            default='python',
            description='Process name to be checked',
            prompt='Process name to be checked',
        )
    ],
    cmd='pgrep {{ input.name }}',
    checking_interval=3
)
runner.register(get_pid_by_name)



from module.auth.component.authorizer import authorizer
from module.auth.component.base import Base
from module.auth.component.password_hasher import password_hasher
from module.auth.component.access_token_scheme import access_token_scheme
from module.auth.component.access_token_util import access_token_util
from module.auth.component.bearer_token_scheme import bearer_token_scheme
from module.auth.component.refresh_token_util import refresh_token_util
from module.auth.component.user import (
    admin_user, admin_user_password, guest_user
)

assert authorizer
assert Base
assert password_hasher
assert access_token_scheme
assert access_token_util
assert bearer_token_scheme
assert refresh_token_util
assert admin_user
assert admin_user_password
assert guest_user



from zrb.builtin import base64
from zrb.builtin import env
from zrb.builtin import eval
from zrb.builtin import git
from zrb.builtin import md5
from zrb.builtin import explain
from zrb.builtin import project
from zrb.builtin import ubuntu
from zrb.builtin import update
from zrb.builtin import version
from zrb.builtin import devtool
from zrb.builtin import generator
from zrb.builtin import process

assert base64
assert env
assert eval
assert git
assert md5
assert explain
assert project
assert ubuntu
assert update
assert version
assert devtool
assert generator
assert process



from typing import Optional
from pydantic import BaseModel
import datetime


class BaseDateTimeSchema(BaseModel):
    created_at: Optional[datetime.datetime]
    created_by: Optional[str]
    updated_at: Optional[datetime.datetime]
    updated_by: Optional[str]


class BaseCountSchema(BaseModel):
    count: int



from zrb.runner import runner
from zrb.task.decorator import python_task
from zrb.task.any_task import AnyTask
from zrb.task.task import Task
from zrb.task.cmd_task import CmdTask
from zrb.task.docker_compose_task import DockerComposeTask, ServiceConfig
from zrb.task.base_remote_cmd_task import BaseRemoteCmdTask, RemoteConfig
from zrb.task.remote_cmd_task import RemoteCmdTask
from zrb.task.rsync_task import RsyncTask
from zrb.task.http_checker import HTTPChecker
from zrb.task.port_checker import PortChecker
from zrb.task.path_checker import PathChecker
from zrb.task.resource_maker import (
    ResourceMaker, Replacement, ReplacementMutator
)
from zrb.task.flow_task import FlowTask
from zrb.task_input.any_input import AnyInput
from zrb.task_input.task_input import Input
from zrb.task_input.bool_input import BoolInput
from zrb.task_input.choice_input import ChoiceInput
from zrb.task_input.float_input import FloatInput
from zrb.task_input.int_input import IntInput
from zrb.task_input.password_input import PasswordInput
from zrb.task_input.str_input import StrInput
from zrb.task_env.env import Env
from zrb.task_env.env_file import EnvFile
from zrb.task_group.group import Group
from zrb.helper.default_env import inject_default_env

assert runner
assert AnyTask
assert python_task
assert Task
assert CmdTask
assert DockerComposeTask
assert ServiceConfig
assert BaseRemoteCmdTask
assert RemoteConfig
assert RemoteCmdTask
assert RsyncTask
assert HTTPChecker
assert PortChecker
assert PathChecker
assert ResourceMaker
assert FlowTask
assert Replacement
assert ReplacementMutator
assert AnyInput
assert Input
assert BoolInput
assert ChoiceInput
assert FloatInput
assert IntInput
assert PasswordInput
assert StrInput
assert Env
assert EnvFile
assert Group

inject_default_env()
